local L = LibStub("AceLocale-3.0"):GetLocale("MCPFu")
local dewdrop = AceLibrary("Dewdrop-2.0")
local tablet = AceLibrary("Tablet-2.0")

MCPFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0")
MCPFu.version = "1.0.0." .. string.sub("$Revision: 5 $", 12, -3)
MCPFu.date = string.sub("$Date: 2010-02-08T20:51:19Z $", 8, 17)
MCPFu.hasIcon = true
MCPFu.tooltipHiddenWhenEmpty = true
MCPFu.clickableTooltip = true

ADDON_LOADED = L["LOADED"];
ADDON_WILL_NOT_LOAD = L["WILL_NOT_LOAD"];
ADDON_REFUSE_TO_LOAD = L["DISABLED_IN_MCP"];

local queuedForDelete = nil;
local lastEquippedSet = nil
local sortedSets = {}
local groupSets = {}
MCPFu_CharVars = {}


local function IsEmpty()
	return next(MCPFu.OnMenuRequest.args.wear.args) == nil
end

local optionsTable = {
	type = 'group',	
	args = {
		add = {
			type = "execute",
			name = L["Add"],
			desc = L["Creates a new set, or updates an existing one."],
			func = "CreateOrUpdateSet",
			order = 101,
		},
		wear = {
			type = "group",
			name = L["Use"],
			desc = L["Use this set of addons."],
			pass = true,
			func = function(key)
				dewdrop:Close()
				MCPFu:UseSet(key)
			end,
			args = {},
			order = 102,
			disabled = IsEmpty
		},
		autoLoad = {
			order = 103,
			type = 'toggle',
			name = L["Auto load"],
			desc = L["Automatically reload UI when sets are changed"],
			get = function()
				return MCPFu.db.profile.autoReloadUI
			end,
			set = function(value)
				MCPFu.db.profile.autoReloadUI = value;				
				MCPFu:UpdateText();
			end,
			disabled = function()
					return false;
			end						
		},								
		spacer = { type = "header", order = 104 },
		delete = {
			type = "group",
			name = L["Delete"],
			desc = L["Delete a set."],
			pass = true,
			func = "DeleteSet",
			args = {},
			order = 107,
			disabled = IsEmpty
		},
		spacer = { type = "header", order = 110 },	
		showSecurity = {
			order = 130,
			type = 'toggle',
			name = L["OPT_SHOW_SECURITY"],
			desc = L["OPT_SHOW_SECURITY_DESC"],
			get = function()
				return MCPFu.db.profile.showSecurity
			end,
			set = function(value)
				MCPFu.db.profile.showSecurity = value;				
			end
		},				
		showNotes = {
			order = 140,
			type = 'toggle',
			name = L["OPT_SHOW_NOTES"],
			desc = L["OPT_SHOW_NOTES_DESC"],
			get = function()
				return MCPFu.db.profile.showNotes
			end,
			set = function(value)
				MCPFu.db.profile.showNotes = value;				
			end
		},		
		maxNoteLength = {
			type = 'range',
			name = L["OPT_NOTE_LEN"],
			desc = L["OPT_NOTE_LEN_DESC"],
			get = function()
				return MCPFu.db.profile.maxChars;
			end,
			set = function(v)
				MCPFu.db.profile.maxChars = v
			end,
			disabled = function()
				if MCPFu.db.profile.showNotes then
					return false;
				else
					return true;
				end
			end,			
			min = 20,
			max = 100,
			step = 5,
			order = 150
		},		
		showEnabled = {
			order = 155,
			type = 'toggle',
			name = L["OPT_SHOW_ENABLED"],
			desc = L["OPT_SHOW_ENABLED_DESC"],
			get = function()
				return MCPFu.db.profile.showEnabled
			end,
			set = function(value)
				MCPFu.db.profile.showEnabled = value;				
				MCPFu:UpdateText();
			end,
			disabled = function()
				if MCPFu.db.profile.showText then
					return false;
				else
					return true;
				end
			end						
		},			
		spacer = { type = "header", order = 165 },		
		groupingCharsStart = {
			type = 'range',
			name = L["Grouping Chars Start"],
			desc = L["The amount of chars to compare at the start of each addon to group the addons together. Recommended to set it to 4, meaning if the first 4 chars of an addon NAME match, they will be grouped together."],
			get = function()
				return MCPFu.db.profile.maxMatchCharacters;
			end,
			set = function(v)
				MCPFu.db.profile.maxMatchCharacters = v
				MCPFu:CreateAddonGroups();	
			end,
			min = 1,
			max = 10,
			step = 1,
			order = 175
		},				
		spacer = { type = "header", order = 185 },
		debuggingLevel = {
			type = 'range',
			name = L["Debug Level"],
			desc = L["Choose level for debugging this tool (-1 = no messages at all) (0 = standard info messages) (>0 for debugging purpose only) "],
			get = function()
				return MCPFu.db.profile.debugLevel
			end,
			set = function(v)
				MCPFu.db.profile.debugLevel = v
			end,
			min = -1,
			max = 5,
			step = 1,
			order = 195
		},						
	}	
}

MCPFu.OnMenuRequest = optionsTable
MCPFu:RegisterChatCommand( { "/MCPFu" }, optionsTable )

MCPFu:RegisterDB("MCPFuDB")
MCPFu:RegisterDefaults('profile', {
	debugLevel = 0,
	showNotes = false,
	maxChars = 20,
	showSecurity = nil,
	showEnabled = 1,
	autoReloadUI = 0,
	maxMatchCharacters = 4,
	ctrlClickHappened = nil,
	set = {},
})

local _G = getfenv(0)

local function reclaim(t)
	if type(t) ~= "table" then return end
	for k, v in pairs(t) do
		if type(v) == "table" then
			t[k] = reclaim(v)
		else
			t[k] = nil
		end
	end
	t = nil
	return nil
end

-- Methods
function MCPFu:OnInitialize()
	self:RegisterDefaults("char", {
		set = {},
	})	

	self.options = optionsTable
	
	-- XXX We're keeping this property for compability for a little while.
	--self.OnMenuRequest = self.options
end

local function updateSortedSets()
	for i in ipairs(sortedSets) do sortedSets[i] = nil end
	for k in pairs(MCPFu.db.profile.set) do table.insert(sortedSets, k) end
	table.sort(sortedSets)
end

function MCPFu:VerifyHiddenAddonsDisabled()
	for i in pairs(MCPFu_CharVars.hiddenAddons) do		
		DisableAddOn(i);
	end	
end
	
function MCPFu:OnEnable()
	if type(_G.StaticPopupDialogs) ~= "table" then
		_G.StaticPopupDialogs = {}
	end
	if not StaticPopupDialogs["MCPFuAdd"] then
		StaticPopupDialogs["MCPFuAdd"] = {
			text = L["Enter the name for your saved Config here !"],
			button1 = L["Add"],
			button2 = L["Cancel"],
			OnCancel = function() self:CancelCreateSet() end,
			sound = "levelup2",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 0,
			OnShow = function()
				-- We have to do this onshow to reset the previous text
				_G[this:GetName().."EditBox"]:SetText("")
			end,
			OnHide = function()
				_G[this:GetName().."EditBox"]:SetText("")
			end,
			EditBoxOnEnterPressed = function()
				local name = _G[this:GetParent():GetName().."EditBox"]:GetText()
				self:CreateSet(name)
				this:GetParent():Hide()
			end,
			EditBoxOnEscapePressed = function()
				self:CancelCreateSet()
				this:GetParent():Hide()
			end,
			OnAccept = function()
				local name = _G[this:GetParent():GetName().."EditBox"]:GetText()
				self:CreateSet(name)
			end,
			hasEditBox = 1,
		}
	end
	
	if not StaticPopupDialogs["MCPFuDelete"] then
		StaticPopupDialogs["MCPFuDelete"] = {
			text = L["Are you sure you want to delete the set %s?"],
			button1 = L["Delete"],
			button2 = L["Cancel"],
			whileDead = 1,
			showAlert = 1,
			timeout = 0,
			OnAccept = function() self:DeleteSet(queuedForDelete, true) end,
			OnCancel = function() queuedForDelete = nil end,
		}
	end	
	
	local function updateLatest(set)
		updateSortedSets()
		self:UpdateDisplay()
	end
	self:RegisterEvent("MCPFu_AddSet", updateLatest)
	self:RegisterEvent("MCPFu_UseSet", updateLatest)
	self:RegisterEvent("MCPFu_DeleteSet", updateLatest)
	
	self:UpdateSetOptions();
	
	self:CreateAddonGroups();	
	
	if (not MCPFu_CharVars.expandedNodes) then
		MCPFu_CharVars.expandedNodes = {};
	end
	if (not MCPFu_CharVars.hiddenAddons) then
		MCPFu_CharVars.hiddenAddons = {};
	end
	
	self:VerifyHiddenAddonsDisabled();
	
	self.db.profile.ctrlClickHappened = nil;
	self.db.profile.altClickHappened = nil;	
	self.db.profile.shiftClickHappened = nil;
end

function MCPFu:UseSet(...)
	name = select(1, ...)
	setToUse = self.db.profile.set[name]
	if type(setToUse) ~= "table" then return false end
	self:TriggerEvent("MCPFu_UseSet", name);	
	
	for k, _ in pairs(setToUse) do
		if (setToUse[k] == 1) then
			EnableAddOn(k);
		else
			DisableAddOn(k);
		end
	end
	
	reclaim(MCPFu_CharVars.expandedNodes);
	MCPFu_CharVars.expandedNodes = {};
	for i in pairs(setToUse.expandedNodes) do
		MCPFu_CharVars.expandedNodes[i] = setToUse.expandedNodes[i];
	end	
	
	if (self.db.profile.autoReloadUI) then
		ReloadUI();		
	end
end

function MCPFu:CancelCreateSet()
	StaticPopup_Hide("MCPFuAdd");
end

function MCPFu:CreateOrUpdateSet(name)
	if UnitAffectingCombat("player") or InCombatLockdown() then return end
	StaticPopup_Show("MCPFuAdd");
end

function MCPFu:CreateSet(name)	
	self:AddSet(name);
	StaticPopup_Hide("MCPFuAdd");
end

function MCPFu:HasSet(name)
	return self.db.profile.set[name] ~= nil
end

function MCPFu:AddSet(name)
	local concatedName = name:gsub(" ", "")
	if not name or name == "" or concatedName == "" then
		self:Print(L["Please use a proper name for your set."])
		return
	end
	if self:HasSet(concatedName) then name = concatedName end

	local text = L["Added set: %s."]:format(name)
	--local event = "ClosetGnome_AddSet"
	if self:HasSet(name) then
		text = L["Updating set: %s."]:format(name)
		--event = "ClosetGnome_UpdateSet"
		reclaim(self.db.profile.set[name]);
	end
	
	--add the set items now
	reclaim(self.db.profile.set[name]);
	self.db.profile.set[name] = {};
	local numAddons = GetNumAddOns();
	for i = 1, numAddons, 1 do		
		local aname, _, _, enabled = GetAddOnInfo(i);
		if (enabled) then
			self.db.profile.set[name][aname] = 1;
		else
			self.db.profile.set[name][aname] = 0;
		end
	end
	reclaim(self.db.profile.set[name].expandedNodes);
	self.db.profile.set[name].expandedNodes = {};
	for i in pairs(MCPFu_CharVars.expandedNodes) do
		self.db.profile.set[name].expandedNodes[i] = MCPFu_CharVars.expandedNodes[i];
	end
	--self.db.profile.set[name][slot] = currentItem
	
	if not self.db.profile.set[name] then return end

	self:TriggerEvent("MCPFu_AddSet", name)
	--self:Print(text)

	self:UpdateSetOptions()
	dewdrop:Close()
end

function MCPFu:DeleteSet(name, verified)
	self:Print("Deleting Test")
	if not verified and not IsShiftKeyDown() then
		queuedForDelete = name
		self:Print("Showing Popup")
		StaticPopup_Show("MCPFuDelete", name)
		return
	end

	queuedForDelete = nil
	if self.db.profile.set[name] then
		self.db.profile.set[name] = reclaim(self.db.profile.set[name])

		self:TriggerEvent("MCPFu_DeleteSet", name)
		self:Print(L["Deleted set: %s."]:format(name))

		self:UpdateSetOptions()
		dewdrop:Refresh()
	end
end

function MCPFu:UpdateSetOptions()
	local opt = self.OnMenuRequest.args

	opt.wear.args = reclaim(opt.wear.args)
	opt.delete.args = reclaim(opt.delete.args)

	opt.wear.args = {}
	opt.delete.args = {}

	for k, v in pairs(self.db.profile.set) do
		opt.wear.args[k] = {
			type = "execute",
			name = k,
			desc = L["Enable the addon set %s."]:format(k),
			icon = icon,
		}
		opt.delete.args[k] = {
			type = "execute",
			name = k,
			desc = L["Deletes the addon set %s."]:format(k),
			icon = icon,
		}
	end
end

function MCPFu:GetAddonGroupStats(groupName)
	local totalAddons =0;
	local enabledAddons = 0;
	local loadedAddons = 0;
	local hiddenAddons = 0;
	if (groupSets[groupName]) then
		for i in pairs(groupSets[groupName]) do
			if (type(groupSets[groupName][i]) == "string") then -- this is an addon string							
				local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(groupSets[groupName][i]);
				if (name) then
					totalAddons = totalAddons +1;
					if (enabled) then
						enabledAddons = enabledAddons +1;
					end
					if (IsAddOnLoaded(groupSets[groupName][i])) then
						loadedAddons = loadedAddons +1;
					end
					if (MCPFu_CharVars.hiddenAddons[groupSets[groupName][i]]) then
						hiddenAddons = hiddenAddons +1;
					end
				end
			end
		end
	end
	return totalAddons, enabledAddons, loadedAddons, hiddenAddons;
end

function MCPFu:CreateAddonGroups()
	local numAddons = GetNumAddOns();
	groupSets = nil;
	groupSets = {};	
	
	groupSets["Auctioneer"] = {};
	groupSets["Auctioneer"].filter = {};
	table.insert(groupSets["Auctioneer"].filter,"Auc");
	table.insert(groupSets["Auctioneer"].filter,"Enchantrix");
	table.insert(groupSets["Auctioneer"].filter,"SlideBar");
	table.insert(groupSets["Auctioneer"].filter,"!Swatter");
	table.insert(groupSets["Auctioneer"].filter,"BeanCounter");
	table.insert(groupSets["Auctioneer"].filter,"Stubby");
	table.insert(groupSets["Auctioneer"].filter,"Informant");
	
	groupSets["FuBar"] = {};
	groupSets["FuBar"].filter = {};
	table.insert(groupSets["FuBar"].filter,"FuBar");
	
	groupSets["Deadly Boss Mods"] = {};
	groupSets["Deadly Boss Mods"].filter = {};
	table.insert(groupSets["Deadly Boss Mods"].filter,"DBM-");	
	
	groupSets["cRange2"] = {};
	groupSets["cRange2"].filter = {};
	table.insert(groupSets["cRange2"].filter,"cRange2");		
	
	groupSets["ORA 2"] = {};
	groupSets["ORA 2"].filter = {};
	table.insert(groupSets["ORA 2"].filter,"oRA2");		
	
	groupSets["Bug Sack"] = {};
	groupSets["Bug Sack"].filter = {};
	table.insert(groupSets["Bug Sack"].filter,"BugSack");			
	table.insert(groupSets["Bug Sack"].filter,"!Bug");			
	
	groupSets["Satrina Buff Frame"] = {};
	groupSets["Satrina Buff Frame"].filter = {};
	table.insert(groupSets["Satrina Buff Frame"].filter,"Satrina");			
	table.insert(groupSets["Satrina Buff Frame"].filter,"SBF");				
	
	groupSets["Bulk Mail 2"] = {};
	groupSets["Bulk Mail 2"].filter = {};
	table.insert(groupSets["Bulk Mail 2"].filter,"BulkMail2");			
	
	groupSets["Libraries"] = {};
	groupSets["Libraries"].filter = {};
	table.insert(groupSets["Libraries"].filter,"Lib");			
	table.insert(groupSets["Libraries"].filter,"LIB");					
	
	groupSets["Shadowed Unit Frames"] = {};
	groupSets["Shadowed Unit Frames"].filter = {};
	table.insert(groupSets["Shadowed Unit Frames"].filter,"Shadowed");			
	
	groupSets["Data Broker Addons"] = {};
	groupSets["Data Broker Addons"].filter = {};
	table.insert(groupSets["Data Broker Addons"].filter,"Ara_Broker");			
	table.insert(groupSets["Data Broker Addons"].filter,"zz");					
	table.insert(groupSets["Data Broker Addons"].filter,"StatBlock");					
	table.insert(groupSets["Data Broker Addons"].filter,"Broker");					
	table.insert(groupSets["Data Broker Addons"].filter,"VertiBroker");					
	table.insert(groupSets["Data Broker Addons"].filter,"Fortress");					
	table.insert(groupSets["Data Broker Addons"].filter,"shPerformance");					
	table.insert(groupSets["Data Broker Addons"].filter,"Wintergrasp Timer");					
	
	
	for i = 1, numAddons, 1 do		
		local name, title = GetAddOnInfo(i);
		local found = false;
		for i in pairs(groupSets) do
			for j in pairs(groupSets[i].filter) do
				local useTitle = false;
				if (title) then						
					self:Debug(name.." - "..title,1);
					if (strfind(title,groupSets[i].filter[j]) == 1) then 
						useTitle = true;
					end
				end
				if (strfind(name,groupSets[i].filter[j]) == 1) or (useTitle == true) then
					--self:Debug(groupSets[i].filter[j],5);
					--self:Debug(name,5);
					table.insert(groupSets[i],name);
					found = true;
				end
			end
		end
		if (found == false) then
			groupSets[name] = {};
			groupSets[name].filter = {};
			table.insert(groupSets[name].filter,string.sub(name,1,self.db.profile.maxMatchCharacters));		
			table.insert(groupSets[name],name);
		end
	end	
	table.sort(groupSets)
end


function MCPFu:Debug(msg,level)
	if (level <= self.db.profile.debugLevel) then
		if (level <= 0) then
			self:Print(msg);
		else
			DEFAULT_CHAT_FRAME:AddMessage(msg);
		end
	end
end

function MCPFu:OnDisable()
end
	
function MCPFu:OnTextUpdate()
	self:SetText("MCP");
	
	if (not self.db.profile.showEnabled) or (not self.db.profile.showText) then
		return;
	end
	
	local numEnabled = 0;
	local numAddons = GetNumAddOns();
	
	for i = 1, numAddons, 1 do				
		name, title, notes, enabled = GetAddOnInfo(i);						
		if (enabled) then
			numEnabled = numEnabled + 1;
		end	
	end
	self:SetText(numEnabled.."/"..numAddons);
end

function MCPFu:OnItemShiftClick(index)
	self:UseSet(index);
end

function MCPFu:OnItemAltClick(index)
	MCPFu_CharVars.hiddenAddons[index] = nil;
end

function MCPFu:OnItemClick(index,enabled) 
	--self:Debug("Checking : "..index,3);
	--if (enabled == nil) then
		--self:Debug("enabled is Nil",3);
		--return;
	--end		
	--if (tpe == nil) then
		--self:Debug("Type is Nil",3);
		--return;
	--end	
	--self:Debug(enabled,3);
	if (enabled ~= "node") then -- its an addon
		if (IsAltKeyDown()) then
			self.db.profile.altClickHappened = 1;
			if (not (index == "FuBar_MCPFu")) then
				MCPFu_CharVars.hiddenAddons[index] = 1;
				DisableAddOn(index);
			end
			return;
		end
		if (IsShiftKeyDown()) then
			self.db.profile.shiftClickHappened = 1;
			UIParentLoadAddOn(index);
			return;
		end
		if (IsControlKeyDown()) then 
			self.db.profile.ctrlClickHappened = 1;
			return;
		end				
		if (not enabled) then
			EnableAddOn(index)
		else
			DisableAddOn(index)
		end
	else -- its a tree node
		if (IsAltKeyDown()) then
			self.db.profile.altClickHappened = 1;
			for j in pairs(groupSets[index]) do				
				if (type(groupSets[index][j]) == "string") then -- this is an addon string													
					MCPFu_CharVars.hiddenAddons[groupSets[index][j]] = nil;
				end
			end			
			return;
		end	
		if (IsControlKeyDown()) then -- collapse all nodes
			self.db.profile.ctrlClickHappened = 1;
			self:Debug(#MCPFu_CharVars.expandedNodes,3);
			if (MCPFu_CharVars.expandedNodes[index] == nil) then -- no nodes expanded			
				for i in pairs(groupSets) do
					--self:Debug(i,3);
					MCPFu_CharVars.expandedNodes[i] = true;
				end
			else
				for i in pairs(MCPFu_CharVars.expandedNodes) do
					--self:Debug(i,3);
					MCPFu_CharVars.expandedNodes[i] = nil;
				end
			end
		elseif (IsShiftKeyDown()) then -- select / deselect all addons
			self.db.profile.shiftClickHappened = 1;
			local enableOrDisable = 2;
			for j in pairs(groupSets[index]) do				
				if (type(groupSets[index][j]) == "string") then -- this is an addon string										
					if (not MCPFu_CharVars.hiddenAddons[groupSets[index][j]]) then
						if (enableOrDisable == 2) then
							_, _, _, enableOrDisable = GetAddOnInfo(groupSets[index][j]);						
						end
						if (not (groupSets[index][j] == "FuBar_MCPFu")) then
							if (not enableOrDisable) then							
								EnableAddOn(groupSets[index][j])
							else
								--self:Debug(groupSets[index][j],3);
								DisableAddOn(groupSets[index][j])
							end
						end
					end
				end
			end
		
		else 
			if (MCPFu_CharVars.expandedNodes[index]) then
				MCPFu_CharVars.expandedNodes[index] = nil;
			else
				MCPFu_CharVars.expandedNodes[index] = 1;
			end
		end
	end
end

function MCPFu:OnTooltipUpdate()
	
	if (IsShiftKeyDown() and (not self.db.profile.shiftClickHappened)) then
		local cat = tablet:AddCategory(
			'columns', 1,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1
		)	
		
		for i in pairs(self.db.profile.set) do
			local line = {};							
			line['func'] = 'OnItemShiftClick'
			line['arg1'] = self			
			line['arg2'] = i -- give the name of the group					
			line['text'] = i;
			
			cat:AddLine(line);
		end
		tablet:SetHint(L["Click the addon set which you want to enable."])		
		return;
	end	
	self.db.profile.shiftClickHappened = nil;


--[[	if (IsControlKeyDown() and (not self.db.profile.ctrlClickHappened)) then
		local cat = tablet:AddCategory(
			'columns', 1,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1
		)	
		return;
	end	]]--
	self.db.profile.ctrlClickHappened = nil;
	
	if (IsAltKeyDown() and (not self.db.profile.altClickHappened)) then
		local cat = tablet:AddCategory(
			'columns', 1,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 1
		)	
		
		local sorting = {};
		for i in pairs(MCPFu_CharVars.hiddenAddons) do table.insert(sorting, i) end		
		table.sort(sorting)
		
		
		for _,i in ipairs(sorting) do
			local line = {};							
			line['func'] = 'OnItemAltClick'
			line['arg1'] = self			
			line['arg2'] = i -- give the name of the group					
			line['text'] = i;
			
			cat:AddLine(line);
			
			tablet:SetHint(L["Click the addon which you want to make visible again."])
		end
		return;
	end	
	self.db.profile.ctrlClickHappened = nil;	
	self.db.profile.altClickHappened = nil;
	
	local cat = tablet:AddCategory(
		'columns', 4,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 1
	)
	
	local sortGroup = {};
	for i in pairs(groupSets) do
		self:Debug(i,5);
		table.insert(sortGroup,i);
	end
	table.sort(sortGroup);
	table.sort(sortGroup, function(a, b)
		return strlower(a) < strlower(b)
	end )
	
	for _,i in ipairs(sortGroup) do
		self:Debug(i,4);
		--line['arg2'] = i		

		if (#groupSets[i] > 1) then -- there is more than 1 addon in this group / filter counts as 1
			local line = {};							
			line['func'] = 'OnItemClick'
			line['arg1'] = self			
			line['arg2'] = i -- give the name of the group			
			line['arg3'] = "node" -- will be overwritten later with checked status
			line['arg4'] = "1" -- type is a node
			line['textR'] = 0.2;
			line['textG'] = 0.2;
			line['textB'] = 1;				
			line['hasCheck'] = true
			line['checked'] = true				
			if (MCPFu_CharVars.expandedNodes[i]) then
				line['checkIcon'] = "Interface\\Buttons\\UI-MinusButton-Up";
			else
				line['checkIcon'] = "Interface\\Buttons\\UI-PlusButton-Up";			
			end
			local tot, enab, load, hidden = self:GetAddonGroupStats(i);
			if (hidden >0) then
				line['text'] = i.." ("..hidden.." hidden)";
			else
				line['text'] = i;
			end
			line['text2'] = enab.."/"..tot.." "..L["enabled"];
			line['text4'] = load.."/"..tot.." "..L["loaded"];
			if (enab+hidden == tot) then
				line['textR'] = 0.2;
				line['textG'] = 1;
				line['textB'] = 0.2;								
			elseif (enab == 0) then
				line['textR'] = 1;
				line['textG'] = 0.2;
				line['textB'] = 0.2;								
			end			
			if (tot ~= hidden) then
				cat:AddLine(line);
			end
		end
		
		if ( (#groupSets[i] == 1) or (MCPFu_CharVars.expandedNodes[i]) ) then -- now show all addons in this group
			for j in pairs(groupSets[i]) do
				self:Debug(groupSets[i][j],3);					
				if ( (type(groupSets[i][j]) == "string") and (not MCPFu_CharVars.hiddenAddons[groupSets[i][j]]) ) then -- this is an addon string					
					--self:Debug(groupSets[i][j],3);
					local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(groupSets[i][j]);
					local line = {};							
					line['func'] = 'OnItemClick'
					line['arg1'] = self								
					line['arg2'] = name -- give the name of the group			
					line['arg3'] = false -- will be overwritten later with checked status					
					
					local loaded = IsAddOnLoaded(name);
					local ondemand = IsAddOnLoadOnDemand(name);
					if (loadable) then
						line['textR'] = 1;
						line['textG'] = 0.78;
						line['textB'] = 0;								
					elseif (enabled and reason ~= "DEP_DISABLED") then
						line['textR'] = 1;
						line['textG'] = 0.1;
						line['textB'] = 0.1;								
					else
						line['textR'] = 0.5;
						line['textG'] = 0.5;
						line['textB'] = 0.5;								
					end
					if (MCPFu_CharVars.expandedNodes[i]) then
						line['text'] = " - ";
					else
						line['text'] = "";
					end
					if (title) then						
						line['text'] = line['text']..title;
					else
						line['text'] = line['text']..name;
					end		
		
					line['hasCheck'] = true												
					line['checkIcon'] = nil;					
					if (enabled) then
						line['checked'] = true			
					else
						line['checked'] = false
					end		
					if (title == "FuBar - |cffffffffMCP|r|cff00ff00Fu|r") then
						line['hasCheck'] = false							
						line['checked'] = false -- make sure it cant be clicked off
					end
					line['arg3'] = line['checked']		
					line['arg4'] = "2" -- type is an addon
		
					if (self.db.profile.showSecurity) then
						if (security == "SECURE") then
							line['text3'] = L["SECURE"];				
						elseif (security == "INSECURE") then
							line['text3'] = L["INSECURE"];				
						elseif (security == "BANNED") then -- wtf?
							line['text3'] = L["BANNED"];				
						end
					end
		
					if (self.db.profile.showNotes) and notes then
						if (string.len(notes) > self.db.profile.maxChars) then
							line['text2'] = string.sub(notes,1,self.db.profile.maxChars-2).."..";				
						else
							line['text2'] = notes;				
						end
					end
		
					if (reason) then			
						line['text4'] = TEXT(getglobal("ADDON_"..reason));				
						if (reason == "DISABLED") or (reason == "WILL_NOT_LOAD") then
							line['checked'] = false				
						end			
					elseif (loaded) then
						line['text4'] = TEXT(ADDON_LOADED);				
					elseif (ondemand) then
						line['text4'] = L["ONDEMAND"];								
						line['text4R'] = 0;
						line['text4G'] = 1;
						line['text4B'] = 0;											
					elseif (enabled) then
						line['text4'] = L["ReloadUI to Load"];												
					else
						line['text4'] = "";												
					end		
		
					cat:AddLine(line)										
				end
			end
		end
	end
	
	tablet:SetHint(L["HINT"])
end
	
function MCPFu:OnClick()
	--PlaySound("SHEATHINGSHIELDSHEATHE");
	if (IsControlKeyDown()) then
		self:CreateAddonGroups();	
	end	
	if (IsAltKeyDown()) then
		ReloadUI();		
	end
	
	--self:CreateAddonGroups();
end

--[[
	local numAddons = GetNumAddOns();
	
	for i = 1, numAddons, 1 do		
		local line = {};	
			
		line['func'] = 'OnItemClick'
		line['arg1'] = self			
		line['arg2'] = i												
		
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i);
		
		--if title and notes then
			--DEFAULT_CHAT_FRAME:AddMessage("Adding : "..title.." - "..notes)
		--end
		local loaded = IsAddOnLoaded(name);
		local ondemand = IsAddOnLoadOnDemand(name);
		if (loadable) then
			line['textR'] = 1;
			line['textG'] = 0.78;
			line['textB'] = 0;								
		elseif (enabled and reason ~= "DEP_DISABLED") then
			line['textR'] = 1;
			line['textG'] = 0.1;
			line['textB'] = 0.1;								
		else
			line['textR'] = 0.5;
			line['textG'] = 0.5;
			line['textB'] = 0.5;								
		end
		if (title) then
			line['text'] = title;
		else
			line['text'] = name;
		end		
		
		line['hasCheck'] = true												
		if (enabled) then
			line['checked'] = true			
		else
			line['checked'] = false
		end		
		if (title == "FuBar - |cffffffffMCP|r|cff00ff00Fu|r") then
			line['hasCheck'] = false							
			line['checked'] = false -- make sure it cant be clicked off
		end
		line['arg3'] = line['checked']		
		
		if (self.db.profile.showSecurity) then
			if (security == "SECURE") then
				line['text3'] = L["SECURE"];				
			elseif (security == "INSECURE") then
				line['text3'] = L["INSECURE"];				
			elseif (security == "BANNED") then -- wtf?
				line['text3'] = L["BANNED"];				
			end
		end
		
		if (self.db.profile.showNotes) and notes then
			if (string.len(notes) > self.db.profile.maxChars) then
				line['text2'] = string.sub(notes,1,self.db.profile.maxChars-2).."..";				
			else
				line['text2'] = notes;				
			end
		end
		
		if (reason) then			
			line['text4'] = TEXT(getglobal("ADDON_"..reason));				
			if (reason == "DISABLED") or (reason == "WILL_NOT_LOAD") then
				line['checked'] = false				
			end			
		elseif (loaded) then
			line['text4'] = TEXT(ADDON_LOADED);				
		elseif (ondemand) then
			line['text4'] = L["ONDEMAND"];								
			line['text4R'] = 0;
			line['text4G'] = 1;
			line['text4B'] = 0;											
		else
			line['text4'] = "";												
		end		
		
		cat:AddLine(line)					
	end
]]--